@@ -21,18 +21,18 @@
 #define ANALYZER_H
 
 #include <QStringList>
+#include <QSharedPointer>
+#include <QStack>
 
 #include "expression.h"
 #include "analitzaexport.h"
 #include "expressiontype.h"
 #include "builtinmethods.h"
-#include <QStack>
 #include <analitza/analitzautils.h>
 
 namespace Analitza
 {
 class Polynomial;
-class Monomial;
 class Apply;
 class BoundingIterator;
 class BuiltinMethods;
@@ -65,13 +65,17 @@ class ANALITZA_EXPORT Analyzer
         /** Constructor. Creates an empty Analyzer module. 
             @param v: Sets a custom variables module. This module will _not_ be deleted along with Analyzer
         */
-        Analyzer(Variables* v);
+        explicit Analyzer(Variables* v);
+        explicit Analyzer(const QSharedPointer<Variables> &v);
         
         /** Copy constructor. Creates a copy of the @p a Analyzer instance. Inherits its Variable structure. */
         Analyzer(const Analyzer& a);
         
         /** Destructor. */
         ~Analyzer();
+
+        /** sets the @p v Variables tuple we are going to use */
+        void setVariables(const QSharedPointer<Variables> &v);
         
         /** Sets an expression to calculate. */
         void setExpression(const Expression &e);
@@ -110,7 +114,7 @@ class ANALITZA_EXPORT Analyzer
         QStringList errors() const { return m_exp.error() + m_err; }
         
         /** @returns Returns a way to query variables. */
-        Variables *variables() const { return m_vars; }
+        QSharedPointer<Variables> variables() const { return m_vars; }
         
         /**
             Adds a variable entry. It is the proper way to do it because tracks some possible errors.
@@ -163,13 +167,12 @@ class ANALITZA_EXPORT Analyzer
         static funcContainer operateContainer[];
         
         Expression m_exp;
-        Variables *m_vars;
+        QSharedPointer<Variables> m_vars;
         QStringList m_err;
         QVector<Object*> m_runStack;
         int m_runStackTop;
         BuiltinMethods m_builtin;
         
-        const bool m_varsOwned;
         bool m_hasdeps;
         ExpressionType m_currentType;
         QMap<QString, ExpressionType> m_variablesTypes;
@@ -212,20 +215,23 @@ class ANALITZA_EXPORT Analyzer
         BoundingIterator* initBVarsContainer(const Apply* n, int base, Object* domain);
         BoundingIterator* initBVarsRange(const Apply* n, int base, Object* dlimit, Object* ulimit);
         
-        template <class T, class Tit, class Tcontained = Object>
+        template <class T, class Tcontained = Object>
         void iterateAndSimp(T* v);
         
         Object* variableValue(Ci* var);
         Object* testResult(const Analitza::Object* o, const QString& var, const Analitza::Object* val);
         
-        template <class T, class Tit, class Tcontained = Object>
+        template <class T, class Tcontained = Object>
         void alphaConversion(T* o, int min);
         void alphaConversion(Apply* a, int min);
         void alphaConversion(Container* a, int min);
         Object* applyAlpha(Analitza::Object* o, int min);
         
-        template<class T, class Tit, class Tcontained = Object>
+        template<class T, class Tcontained = Object>
         Object* calcElements(const Analitza::Object* root, T* nv);
+
+        template<class T, class Tcontained = Object>
+        Object* evalElements(const Analitza::Object* root, T* nv, bool resolve, const QSet<QString>& unscoped);
 };
 
 }
